*Estimating supply and demand elasticities in STATA
*Fabio Gaetano Santeramo
*fabiogaetano.santeramo@gmail.com

*						Section 1 . Producer theory								*
*	This example considers data for input use, output, quantities, costs, and 	*
*	prices for total U.S. nondurable manufacturing for 1949-2001. 				*
*					We use the file Dataset1.csv.								*
							
clear all
set mem 10000K
insheet using "<write your path here>\Dataset1.csv"

*********************************************************************************
*						Variable generation & data setup						*
*********************************************************************************

*Generate log variables
gen q = log(y_q)
gen pk = log(k_p)
gen pl = log(l_p)
gen pe = log(e_p)
gen pm = log(m_p)
gen ps = log(s_p)

*Generate sum of costs
gen c = k_v + l_v + e_v + m_v + s_v

*Generate cost shares
gen k_s = k_v/c
gen l_s = l_v/c
gen e_s = e_v/c
gen m_s = m_v/c
gen s_s = s_v/c

gen lc = log(c)

*********************************************************************************
*					1.1 Estimate the double-log factor demands					*
*********************************************************************************
regr k_s pk pl pe pm ps q
regr l_s pk pl pe pm ps q
regr e_s pk pl pe pm ps q
regr m_s pk pl pe pm ps q

*********************************************************************************
*		1.2 Estimate the translog factor demands without a cost function, 		*
*				also imposing homogeneity and symmetry.							*
*********************************************************************************

*SUR estimation - UNCONSTRAINED
sureg (k_s pk pl pe pm ps q) (l_s pk pl pe pm ps q) (e_s pk pl pe pm ps q) ///
(m_s pk pl pe pm ps q), corr
*Tests for HOMOGENEITY
test ([k_s]ps = 0 - ([k_s]pk + [k_s]pl + [k_s]pe + [k_s]pm)) ///
([l_s]ps = 0 - ([l_s]pk + [l_s]pl + [l_s]pe + [l_s]pm)) ///
([e_s]ps = 0 - ([e_s]pk + [e_s]pl + [e_s]pe + [e_s]pm)) ///
([m_s]ps = 0 - ([m_s]pk + [m_s]pl + [m_s]pe + [m_s]pm))
*Tests for SYMMETRY
test ([k_s]pl-[l_s]pk=0) ([k_s]pe-[e_s]pk=0) ([k_s]pm-[m_s]pk=0) ///
([l_s]pm-[m_s]pl=0) ([e_s]pm-[m_s]pe=0) ([l_s]pe-[e_s]pl=0)

*We can also jointly test for HOMOGENEITY AND SYMMETRY using LR Test
**SUR estimation - UNCONSTRAINED
sureg (k_s pk pl pe pm ps q) (l_s pk pl pe pm ps q) (e_s pk pl pe pm ps q) ///
(m_s pk pl pe pm ps q), corr 
**Save results
est store unrestr
**SUR estimation  CONSTRAINED
**Define constraints
constraint 1  [k_s]ps = 0 - ([k_s]pk + [k_s]pl + [k_s]pe + [k_s]pm)
constraint 2  [l_s]ps = 0 - ([l_s]pk + [l_s]pl + [l_s]pe + [l_s]pm)
constraint 3  [e_s]ps = 0 - ([e_s]pk + [e_s]pl + [e_s]pe + [e_s]pm)
constraint 4  [m_s]ps = 0 - ([m_s]pk + [m_s]pl + [m_s]pe + [m_s]pm)
constraint 5  [k_s]pl-[l_s]pk=0 
constraint 6  [k_s]pe-[e_s]pk=0 
constraint 7  [k_s]pm-[m_s]pk=0 
constraint 8  [l_s]pm-[m_s]pl=0 
constraint 9  [e_s]pm-[m_s]pe=0 
constraint 10 [l_s]pe-[e_s]pl=0
**Estimate the model with constraints
sureg (k_s pk pl pe pm ps q) (l_s pk pl pe pm ps q) (e_s pk pl pe pm ps q) ///
(m_s pk pl pe pm ps q), constraints(1 2 3 4 5 6 7 8 9 10) corr
**Save results
est store restr
**Test for homogeneity and symmetry using the LR test
lrtest restr unrestr

*********************************************************************************
*		1.3 Estimate the translog factor demands including the cost function, 	*
*				and imposing homogeneity and symmetry.							*
*********************************************************************************

*Generate squared variables 
g q2    = q*q
g pk2  = pk*pk
g pl2   = pl*pl
g pe2  = pe*pe
g pm2 = pm*pm
g ps2   = ps*ps

*Generate cross-products variables
g pspk  = ps*pk
g pspl   = ps*pl
g pspe  = ps*pe
g pspm= ps*pm

g pkps   = pk*ps
g pkpl   = pk*pl
g pkpe  = pk*pe
g pkpm = pk*pm
g plps    = pl*ps
g plpk   = pl*pk
g plpe   = pl*pe
g plpm  = pl*pm

g peps  = pe*ps
g pepl   = pe*pl
g pepk  = pe*pk
g pepm = pe*pm

g pmps = pm*ps
g pmpl  = pm*pl
g pmpe = pm*pe
g pmpk = pm*pk

g qps   = q*ps
g qpl   = q*pl
g qpe  = q*pe
g qpk  = q*pk
g qpm = q*pm

*Define additional constraints on cross-products
constraint 11 ps2 = 0-(pkps + pspl + peps + pspm)
constraint 12 qps= 0-(qpk + qpl + qpe + qpm)
constraint 13 ps  = 1-(pk + pl + pe + pm)  

*Estimate the system of equations imposing constraints for homogeneity and symmetry
sureg (lc q pk pl pe pm ps q2 pk2 pl2 pe2 pm2 ps2 pspl pspm pkps pkpl pkpe ///
pkpm plpm  peps pepl pepm qps qpl qpe qpk qpm) (k_s pk pl pe pm ps q) ///
(l_s pk pl pe pm ps q) (e_s pk pl pe pm ps q) (m_s pk pl pe pm ps q), ///
constraints(1 2 3 4 11 12 13 ) corr

*********************************************************************************
*					1.4 Compute elasticities of interest						*
*********************************************************************************			

*Define the matrix of variables averages
qui sum k_s
scalar mu_k_s =  r(mean)   

qui sum l_s
scalar mu_l_s =  r(mean)

qui sum e_s
scalar mu_e_s =  r(mean)

qui sum m_s
scalar mu_m_s =  r(mean)

qui sum s_s
scalar mu_s_s =  r(mean)

matrix Mean = J(5,1,.)
matrix Mean[1,1] = mu_k_s
matrix Mean[2,1] = mu_l_s
matrix Mean[3,1] = mu_e_s
matrix Mean[4,1] = mu_m_s
matrix Mean[5,1] = mu_s_s
matrix list Mean

*Estimate and save the results of the system of equations 
sureg (lc q pk pl pe pm ps q2 pk2 pl2 pe2 pm2 ps2 pspl pspm pkps pkpl pkpe pkpm plpm  peps pepl pepm qps qpl qpe qpk qpm) (k_s pk pl pe pm ps q) (l_s pk pl pe pm ps q) (e_s pk pl pe pm ps q) (m_s pk pl pe pm ps q), constraints (1 2 3 4 11 12 13 ) corr
est store estimate

*Save ALL coefficients in a matrix
matrix bvec = e(b)
matrix list bvec

*Compute scalars
scalar gks = 0 - (_b[k_s:pk] + _b[k_s:pl] + _b[k_s:pe] + _b[k_s:pm])
scalar gls = 0 - (_b[k_s:pl] + _b[l_s:pl] + _b[l_s:pe] + _b[l_s:pm])
scalar ges = 0 - (_b[k_s:pe] + _b[l_s:pe] + _b[e_s:pe] + _b[e_s:pm])
scalar gms = 0 - (_b[k_s:pm] + _b[l_s:pm] + _b[e_s:pm] + _b[m_s:pm])
scalar gss = 0 - (_b[k_s:ps] + _b[l_s:ps] + _b[e_s:ps] + _b[m_s:ps])

*Define the matrix of coefficients
matrix gij=(_b[k_s:pk], _b[k_s:pl], _b[k_s:pe], _b[k_s:pm], _b[k_s:ps]\_b[k_s:pl], _b[l_s:pl], _b[l_s:pe], _b[l_s:pm], _b[l_s:ps]\_b[k_s:pe], _b[l_s:pe], _b[e_s:pe], _b[e_s:pm], _b[e_s:ps]\_b[k_s:pm], _b[l_s:pm], _b[e_s:pm], _b[m_s:pm], _b[m_s:ps]\_b[k_s:ps], _b[l_s:ps], _b[e_s:ps], _b[m_s:ps], gss)

matrix eos = J(5,5,.)
matrix list eos

matrix mos = J(5,5,.)
matrix list mos


matrix ep  = J(5,5,.)
matrix list ep

*Define an identity matrix
matrix I = -I(5)
matrix lis I

*Compute Elasticities of substitution and price elasticites
forvalues i = 1/5  {
forvalues j = 1/5  {
matrix eos[`i',`j'] = (gij[`i',`j']+Mean[`i',1]*Mean[`j',1]+I[`i',`j']*Mean[`i',1])/(Mean[`i',1]*Mean[`j',1])
matrix ep[`i',`j']= (Mean[`i',1]*eos[`i',`j'])
}
}

*Compute Morishima elasticities 
forvalues i = 1/5  {
forvalues j = 1/5  {
matrix mos[`i',`j'] = (ep[`i',`j']-ep[`j',`i'])
}
}

*Show the matrices
mat lis ep
mat lis eos
mat lis mos
