*Estimating supply and demand elasticities in STATA
*Fabio Gaetano Santeramo
*fabiogaetano.santeramo@gmail.com

*						Section 2 . Consumer theory								*
*	This consumer theory example shows how to estimate a system of				*
*	demand functions by using the absolute version of the Rotterdam Model.		*
*					We use the file Dataset2.csv								*
							
clear all
set mem 10000K
insheet using "<write your path here>\Dataset2.csv"

*********************************************************************************
*						Variable generation	& data setup						*
*********************************************************************************

*# QUANTITIES
gen log_bfvlq= log(bfvlq)
gen log_porkq = log(porkq)
gen log_poultq= log(poultq)
gen log_fishq = log(fishq)

*# PRICES
gen log_bfvlp= log(bfvlp)
gen log_porkp = log(porkp)
gen log_poultp= log(poultp)
gen log_fishp = log(fishp)

gen log_xtotal=log(xtotal)

*Define the time series nature of data
tset year

*Generate deltalog variables
gen dlog_bfvlq= d.log_bfvlq
gen dlog_porkq = d.log_porkq
gen dlog_poultq= d.log_poultq
gen dlog_fishq = d.log_fishq

gen dlog_bfvlp= d.log_bfvlp
gen dlog_porkp = d.log_porkp
gen dlog_poultp= d.log_poultp
gen dlog_fishp = d.log_fishp

gen dlog_xtotal=d.log_xtotal

*Generate the variable W hat 
gen xbfvl_hat        = 0.5*(xbfvl  +l.xbfvl)
gen xpork_hat      = 0.5*(xpork  +l.xpork)
gen xpoultry_hat = 0.5*(xpoult +l.xpoult)
gen xfish_hat        = 0.5*(xfish  +l.xfish)

*Generate the dependent variables(w )
gen wdlog_bfvlq = dlog_bfvlq*xbfvl_hat
gen wdlog_porkq  =  dlog_porkq*xpork_hat
gen wdlog_poultq =  dlog_poultq*xpoultry_hat
gen wdlog_fishq    =  dlog_fishq*xfish_hat

*Generate real income variables
gen real_bfvl    = dlog_xtotal-(dlog_porkp*xpork_hat  + dlog_poultp*xpoultry_hat ///
+ dlog_fishp*xfish_hat ) 
gen real_pork  = dlog_xtotal-(dlog_bfvlp*xbfvl_hat + dlog_poultp*xpoultry_hat + ///
dlog_fishp*xfish_hat ) 
gen real_poult = dlog_xtotal-(dlog_porkp*xpork_hat + dlog_bfvlp*xbfvl_hat + ///
dlog_fishp*xfish_hat  )
gen real_fish    = dlog_xtotal-(dlog_porkp*xpork_hat + dlog_poultp*xpoultry_hat + ///
dlog_bfvlp*xbfvl_hat)

*********************************************************************************
*					2.1 Regressions we need to estimate							*
*********************************************************************************		

regr wdlog_bfvlq real_bfvl dlog_bfvlp dlog_porkp dlog_poultp dlog_fishp
regr wdlog_porkq  real_pork dlog_bfvlp dlog_porkp dlog_poultp dlog_fishp
regr wdlog_poultq real_poult dlog_bfvlp dlog_porkp dlog_poultp dlog_fishp
regr wdlog_fishq real_fish dlog_bfvlp dlog_porkp dlog_poultp dlog_fishp

*********************************************************************************
*		2.2 Estimate the Rotterdam model(where one equation has been dropped) 	*
*					and test for homogeneity and symmetry.						*
*********************************************************************************

*Estimate the unrestricted model
sureg ( wdlog_bfvlq real_bfvl dlog_bfvlp dlog_porkp dlog_poultp) ///
(wdlog_porkq real_pork dlog_bfvlp dlog_porkp dlog_poultp) ///
(wdlog_poultq real_poult dlog_bfvlp dlog_porkp dlog_poultp)

*Save the results
est store unrestr

*Show the Likelihood
di e(ll)

*Define the constraint for homogeneity
constraint 1 [wdlog_bfvlq]dlog_bfvlp = 0 - ([wdlog_porkq]dlog_porkp + ///
[wdlog_poultq]dlog_poultp)

*Define the constraints for symmetry
constraint 2 [wdlog_bfvlq]dlog_porkp    -   [wdlog_porkq]dlog_bfvlp   = 0
constraint 3 [wdlog_porkq]dlog_poultp - [wdlog_poultq]dlog_porkp  = 0
constraint 4 [wdlog_bfvlq]dlog_poultp  -  [wdlog_poultq]dlog_bfvlp   = 0

*Estimate the restricted model, save the results, and show the Likelihood
sureg ( wdlog_bfvlq real_bfvl dlog_bfvlp dlog_porkp dlog_poultp) ///
(wdlog_porkq real_pork dlog_bfvlp dlog_porkp dlog_poultp) ///
(wdlog_poultq real_poult dlog_bfvlp dlog_porkp dlog_poultp), ///
constraints(1 2 3 4)

est store restr

di e(ll)

*Compute the likelihood ratio test
lrtest restr unrestr

*Alternatively, we can compute the LR test as follows: 
*	LL unrest:-312.12
*	LL restr: -341.05
*	LR: -2*(LL restr-LL unrest)
*	R: -2*(-341.05-(-312.12))

*********************************************************************************
*			2.3 In order to compute the elasticities, we estimate the 			*
*		model imposing the restrictions and use the prices to compute the 		*
*		expenditure elasticities, the compensated elasticities and the 			*
*						uncompensated elasticities. 							*
*********************************************************************************

*Generate a scalar for prices
gen P_bv = 0.98
gen P_pork  = 0.77
gen P_poul  = 0.56
gen P_fish= 1.01

*Compute the expenditure elasticities
**Expenditure elasticity  for beef and veal
di [wdlog_bfvlq]real_bfvl/P_bv               
**Expenditure elasticity  for pork
di [wdlog_porkq]real_pork/P_pork     
**Expenditure elasticity  for poultry
di [wdlog_poultq]real_poult/P_poul   
**Expenditure elasticity  for fish
di 1-(0.46+0.13-0.15)

*Define the sample size
gen obs=41

*Generate average variables
gen mean_wbfvl   = xbfvl/obs
gen mean_pork     = xpork/obs
gen mean_poult    = xpoult/obs
gen mean_fish       = xfish/obs

*Compute the compensated elasticities
di [wdlog_bfvlq]dlog_bfvlp/mean_wbfvl
di [wdlog_bfvlq]dlog_porkp/mean_wbfvl
di [wdlog_bfvlq]dlog_poult/mean_wbfvl

di [wdlog_porkq]dlog_porkp/mean_pork
di [wdlog_porkq]dlog_bfvlp/mean_pork
di [wdlog_porkq]dlog_poult/mean_pork

di [wdlog_poultq]dlog_poultp/mean_poult
di [wdlog_poultq]dlog_bfvlp/mean_poult
di [wdlog_poultq]dlog_porkp/mean_poult

*Generate the compensated elasticities
gen e_c_bf_bf     = [wdlog_bfvlq]dlog_bfvlp/mean_wbfvl
gen e_c_bf_por   =  [wdlog_bfvlq]dlog_porkp/mean_wbfvl
gen e_c_bf_poul = [wdlog_bfvlq]dlog_poult/mean_wbfvl

gen e_c_por_por = [wdlog_porkq]dlog_porkp/mean_pork
gen e_c_por_bf    = [wdlog_porkq]dlog_bfvlp/mean_pork
gen e_c_por_poul= [wdlog_porkq]dlog_poult/mean_pork

gen e_c_poul_poul= [wdlog_poultq]dlog_poultp/mean_poult
gen e_c_poul_bf    = [wdlog_poultq]dlog_bfvlp/mean_poult
gen e_c_poul_por = [wdlog_poultq]dlog_porkp/mean_poult

*Generate expenditure elasticities
gen e_bv  =  [wdlog_bfvlq]real_bfvl/P_bv 
gen e_pork=  [wdlog_porkq]real_pork/P_pork 
gen e_poult= [wdlog_poultq]real_poult/P_poul

*Generate the uncompensated elasticities
gen e_uc_bf_por   =  e_c_bf_por  - mean_pork*e_bv
gen e_uc_bf_poul = e_c_bf_poul - mean_poult*e_bv

gen e_uc_por_bf    =   e_c_por_bf  - mean_wbfvl*e_pork
gen e_uc_por_poul= e_c_por_poul- mean_poult*e_pork

gen e_uc_poul_bf    =   e_c_poul_bf  - mean_wbfvl*e_poult
gen e_uc_poul_por = e_c_poul_por - mean_pork*e_poult


